@@ -33,11 +33,11 @@ static std::mutex g_clangMutex;
 ClangParser *ClangParser::instance()
 {
   std::lock_guard<std::mutex> lock(g_clangMutex);
-  if (!s_instance) s_instance = new ClangParser;
+  if (s_instance==nullptr) s_instance = new ClangParser;
   return s_instance;
 }
 
-ClangParser *ClangParser::s_instance = 0;
+ClangParser *ClangParser::s_instance = nullptr;
 
 //--------------------------------------------------------------------------
 #if USE_LIBCLANG
@@ -72,7 +72,7 @@ class ClangTUParser::Private
       : parser(p), fileDef(fd) {}
     const ClangParser &parser;
     const FileDef *fileDef;
-    CXIndex index = 0;
+    CXIndex index = nullptr;
     uint32_t curToken = 0;
     DetectedLang detectedLang = DetectedLang::Cpp;
     size_t numFiles = 0;
@@ -80,14 +80,15 @@ class ClangTUParser::Private
     std::vector<CXUnsavedFile> ufs;
     std::vector<CXCursor> cursors;
     std::unordered_map<std::string,uint32_t> fileMapping;
-    CXTranslationUnit tu = 0;
-    CXToken *tokens = 0;
+    CXTranslationUnit tu = nullptr;
+    CXToken *tokens = nullptr;
     uint32_t numTokens = 0;
     StringVector filesInSameTU;
     TooltipManager tooltipManager;
+    std::vector<const Definition *> foldStack;
 
     // state while parsing sources
-    const MemberDef  *currentMemberDef=0;
+    const MemberDef  *currentMemberDef=nullptr;
     uint32_t          currentLine=0;
     bool              searchForBody=FALSE;
     bool              insideBody=FALSE;
@@ -119,8 +120,8 @@ void ClangTUParser::parse()
   const StringVector &clangOptions = Config_getList(CLANG_OPTIONS);
   if (!clangAssistedParsing) return;
   //printf("ClangParser::start(%s)\n",fileName);
-  assert(p->index==0);
-  assert(p->tokens==0);
+  assert(p->index==nullptr);
+  assert(p->tokens==nullptr);
   assert(p->numTokens==0);
   p->index    = clang_createIndex(0, 0);
   p->curToken = 0;
@@ -140,6 +141,22 @@ void ClangTUParser::parse()
     {
       argv.push_back(qstrdup(option->c_str()));
     }
+    // The last compile command (last entry of argv) should be the filename of the source
+    // file to parse. It does not matter to clang_parseTranslationUnit below if we pass the file name
+    // separately in its second argument or if we just pass it a nullptr as the second
+    // argument and pass the file name with the other compile commands.
+    // However, in some cases (e.g., starting from Clang 14, if we are parsing a header file, see
+    // https://github.com/doxygen/doxygen/issues/10733), the compile commands returned by
+    // getCompileCommands include a "--" as second to last argument (which is supposed to make it
+    // easier to parse the argument list). If we pass this "--" to clang_parseTranslationUnit below,
+    // it returns an error. To avoid this, we remove the file name argument (and the "--" if present)
+    // from argv and pass the file name separately.
+    argv.pop_back(); // remove file name
+    if (std::string(argv[argv.size() - 1]) == "--") {
+      // remove '--' from argv
+      argv.pop_back();
+    }
+
     // user specified options
     for (size_t i=0;i<clangOptions.size();i++)
     {
@@ -180,9 +197,9 @@ void ClangTUParser::parse()
     // we use the source file to detected the language. Detection will fail if you
     // pass a bunch of .h files containing ObjC code, and no sources :-(
     SrcLangExt lang = getLanguageFromFileName(fileName);
-    if (lang==SrcLangExt_ObjC || p->detectedLang!=DetectedLang::Cpp)
+    QCString fn = fileName.lower();
+    if (lang==SrcLangExt::ObjC || p->detectedLang!=DetectedLang::Cpp)
     {
-      QCString fn = fileName.lower();
       if (p->detectedLang!=DetectedLang::Cpp &&
           (fn.endsWith(".cpp") || fn.endsWith(".cxx") ||
            fn.endsWith(".cc")  || fn.endsWith(".c")))
@@ -200,14 +217,16 @@ void ClangTUParser::parse()
     }
     switch (p->detectedLang)
     {
-      case DetectedLang::Cpp:    argv.push_back(qstrdup("c++"));           break;
+      case DetectedLang::Cpp:
+        if (fn.endsWith(".hpp") || fn.endsWith(".hxx") ||
+            fn.endsWith(".hh")  || fn.endsWith(".h"))
+          argv.push_back(qstrdup("c++-header"));
+        else
+          argv.push_back(qstrdup("c++"));
+        break;
       case DetectedLang::ObjC:   argv.push_back(qstrdup("objective-c"));   break;
       case DetectedLang::ObjCpp: argv.push_back(qstrdup("objective-c++")); break;
     }
-
-    // provide the input and its dependencies as unsaved files so we can
-    // pass the filtered versions
-    argv.push_back(qstrdup(fileName.data()));
   }
   //printf("source %s ----------\n%s\n-------------\n\n",
   //    fileName,p->source.data());
@@ -215,7 +234,7 @@ void ClangTUParser::parse()
   p->numFiles = numUnsavedFiles;
   p->sources.resize(numUnsavedFiles);
   p->ufs.resize(numUnsavedFiles);
-  int refIndent = 0;
+  size_t refIndent = 0;
   p->sources[0]      = detab(fileToString(fileName,filterSourceFiles,TRUE),refIndent);
   p->ufs[0].Filename = qstrdup(fileName.data());
   p->ufs[0].Contents = p->sources[0].data();
@@ -235,7 +254,7 @@ void ClangTUParser::parse()
 
   // let libclang do the actual parsing
   //for (i=0;i<argv.size();i++) printf("Argument %d: %s\n",i,argv[i]);
-  p->tu = clang_parseTranslationUnit(p->index, 0,
+  p->tu = clang_parseTranslationUnit(p->index, fileName.data(),
                                      argv.data(), static_cast<int>(argv.size()), p->ufs.data(), numUnsavedFiles,
                                      CXTranslationUnit_DetailedPreprocessingRecord);
   //printf("  tu=%p\n",p->tu);
@@ -277,7 +296,7 @@ ClangTUParser::~ClangTUParser()
     clang_disposeTranslationUnit(p->tu);
     clang_disposeIndex(p->index);
     p->fileMapping.clear();
-    p->tokens    = 0;
+    p->tokens    = nullptr;
     p->numTokens = 0;
   }
   for (size_t i=0;i<p->numFiles;i++)
@@ -287,7 +306,7 @@ ClangTUParser::~ClangTUParser()
   p->ufs.clear();
   p->sources.clear();
   p->numFiles  = 0;
-  p->tu        = 0;
+  p->tu        = nullptr;
 }
 
 void ClangTUParser::switchToFile(const FileDef *fd)
@@ -297,7 +316,7 @@ void ClangTUParser::switchToFile(const FileDef *fd)
   {
     p->cursors.clear();
     clang_disposeTokens(p->tu,p->tokens,p->numTokens);
-    p->tokens    = 0;
+    p->tokens    = nullptr;
     p->numTokens = 0;
 
     CXFile f = clang_getFile(p->tu, fd->absFilePath().data());
@@ -326,18 +345,18 @@ std::string ClangTUParser::lookup(uint32_t line,const char *symbol)
 {
   //printf("ClangParser::lookup(%d,%s)\n",line,symbol);
   std::string result;
-  if (symbol==0) return result;
+  if (symbol==nullptr) return result;
   bool clangAssistedParsing = Config_getBool(CLANG_ASSISTED_PARSING);
   if (!clangAssistedParsing) return result;
 
-  auto getCurrentTokenLine = [=]() -> uint32_t
+  auto getCurrentTokenLine = [this]() -> uint32_t
   {
-    uint32_t l, c;
+    uint32_t l=0, c=0;
     if (p->numTokens==0) return 1;
     // guard against filters that reduce the number of lines
     if (p->curToken>=p->numTokens) p->curToken=p->numTokens-1;
     CXSourceLocation start = clang_getTokenLocation(p->tu,p->tokens[p->curToken]);
-    clang_getSpellingLocation(start, 0, &l, &c, 0);
+    clang_getSpellingLocation(start, nullptr, &l, &c, nullptr);
     return l;
   };
 
@@ -384,7 +403,7 @@ std::string ClangTUParser::lookup(uint32_t line,const char *symbol)
         ts = clang_getCString(tokenString);
         tl = ts ? strlen(ts) : 0;
         // skip over any spaces in the symbol
-        char c;
+        char c = 0;
         while (offset<sl && ((c=symbol[offset])==' ' || c=='\t' || c=='\r' || c=='\n'))
         {
           offset++;
@@ -429,14 +448,80 @@ std::string ClangTUParser::lookup(uint32_t line,const char *symbol)
   return result;
 }
 
+void ClangTUParser::codeFolding(OutputCodeList &ol,const Definition *d,uint32_t line)
+{
+  if (Config_getBool(HTML_CODE_FOLDING))
+  {
+    endCodeFold(ol,line);
+    if (d)
+    {
+      int startLine = d->getStartDefLine();
+      int endLine   = d->getEndBodyLine();
+      if (endLine!=-1 && startLine!=endLine &&
+          // since the end of a section is closed after the last line, we need to avoid starting a
+          // new section if the previous section ends at the same line, i.e. something like
+          // struct X {
+          // ...
+          // }; struct S {  <- start of S and end of X at the same line
+          // ...
+          // };
+          (p->foldStack.empty() || p->foldStack.back()->getEndBodyLine()!=startLine))
+      {
+        if (d->definitionType()==Definition::TypeMember)
+        {
+          const MemberDef *md = toMemberDef(d);
+          if (md && md->isDefine())
+          {
+            ol.startFold(line,"",""); // #define X ...
+          }
+          else if (md && md->isCallable())
+          {
+            ol.startFold(line,"{","}"); // func() { ... }
+          }
+          else
+          {
+            ol.startFold(line,"{","};"); // enum X { ... }
+          }
+        }
+        else if (d->definitionType()==Definition::TypeClass)
+        {
+          ol.startFold(line,"{","};"); // class X { ... };
+        }
+        else
+        {
+          ol.startFold(line,"{","}"); // namespace X {...}
+        }
+        p->foldStack.push_back(d);
+      }
+    }
+  }
+}
+
+void ClangTUParser::endCodeFold(OutputCodeList &ol,uint32_t line)
+{
+  while (!p->foldStack.empty())
+  {
+    const Definition *dd = p->foldStack.back();
+    if (dd->getEndBodyLine()+1==static_cast<int>(line))
+    {
+      ol.endFold();
+      p->foldStack.pop_back();
+    }
+    else
+    {
+      break;
+    }
+  }
+}
 
 void ClangTUParser::writeLineNumber(OutputCodeList &ol,const FileDef *fd,uint32_t line,bool writeLineAnchor)
 {
-  const Definition *d = fd ? fd->getSourceDefinition(line) : 0;
-  if (d && fd->isLinkable())
+  const Definition *d = fd ? fd->getSourceDefinition(line) : nullptr;
+  if (d)
   {
     p->currentLine=line;
     const MemberDef *md = fd->getSourceMember(line);
+    //printf("writeLineNumber(%p,line=%d)\n",(void*)md,line);
     if (md && md->isLinkable())  // link to member
     {
       if (p->currentMemberDef!=md) // new member, start search for body
@@ -446,31 +531,39 @@ void ClangTUParser::writeLineNumber(OutputCodeList &ol,const FileDef *fd,uint32_
         p->bracketCount=0;
       }
       p->currentMemberDef=md;
+      codeFolding(ol,md,line);
       ol.writeLineNumber(md->getReference(),
                          md->getOutputFileBase(),
                          md->anchor(),
                          line,writeLineAnchor);
     }
-    else // link to compound
+    else if (d->isLinkable()) // link to compound
     {
-      p->currentMemberDef=0;
+      p->currentMemberDef=nullptr;
+      codeFolding(ol,d,line);
       ol.writeLineNumber(d->getReference(),
                          d->getOutputFileBase(),
                          d->anchor(),
                          line,writeLineAnchor);
     }
+    else // no link
+    {
+      codeFolding(ol,nullptr,line);
+      ol.writeLineNumber(QCString(),QCString(),QCString(),line,writeLineAnchor);
+    }
   }
   else // no link
   {
+    codeFolding(ol,nullptr,line);
     ol.writeLineNumber(QCString(),QCString(),QCString(),line,writeLineAnchor);
   }
 
   // set search page target
-  if (Doxygen::searchIndex)
+  if (Doxygen::searchIndex.enabled())
   {
     QCString lineAnchor;
     lineAnchor.sprintf("l%05d",line);
-    Doxygen::searchIndex->setCurrentDoc(fd,lineAnchor,TRUE);
+    Doxygen::searchIndex.setCurrentDoc(fd,lineAnchor,TRUE);
   }
 
   //printf("writeLineNumber(%d) g_searchForBody=%d\n",line,g_searchForBody);
@@ -481,7 +574,7 @@ void ClangTUParser::codifyLines(OutputCodeList &ol,const FileDef *fd,const char
 {
   if (fontClass) ol.startFontClass(fontClass);
   const char *p=text,*sp=p;
-  char c;
+  char c = 0;
   bool inlineCodeFragment = false;
   bool done=FALSE;
   while (!done)
@@ -491,17 +584,14 @@ void ClangTUParser::codifyLines(OutputCodeList &ol,const FileDef *fd,const char
     if (c=='\n')
     {
       line++;
-      int l = static_cast<int>(p-sp-1);
+      size_t l = static_cast<size_t>(p-sp-1);
       column=l+1;
-      char *tmp = static_cast<char *>(malloc(l+1));
-      memcpy(tmp,sp,l);
-      tmp[l]='\0';
-      ol.codify(tmp);
-      free(tmp);
+      std::string tmp(sp,l);
+      ol.codify(tmp.c_str());
       if (fontClass) ol.endFontClass();
       ol.endCodeLine();
-      ol.startCodeLine(line);
       writeLineNumber(ol,fd,line,inlineCodeFragment);
+      ol.startCodeLine(line);
       if (fontClass) ol.startFontClass(fontClass);
     }
     else
@@ -534,7 +624,7 @@ void ClangTUParser::writeMultiLineCodeLink(OutputCodeList &ol,
   while (!done)
   {
     const char *sp=p;
-    char c;
+    char c = 0;
     while ((c=*p++) && c!='\n') { column++; }
     if (c=='\n')
     {
@@ -542,8 +632,8 @@ void ClangTUParser::writeMultiLineCodeLink(OutputCodeList &ol,
       //printf("writeCodeLink(%s,%s,%s,%s)\n",ref,file,anchor,sp);
       ol.writeCodeLink(d->codeSymbolType(),ref,file,anchor,QCString(sp,p-sp-1),tooltip);
       ol.endCodeLine();
-      ol.startCodeLine(line);
       writeLineNumber(ol,fd,line,inlineCodeFragment);
+      ol.startCodeLine(line);
     }
     else
     {
@@ -559,7 +649,7 @@ void ClangTUParser::linkInclude(OutputCodeList &ol,const FileDef *fd,
 {
   QCString incName = text;
   incName = incName.mid(1,incName.length()-2); // strip ".." or  <..>
-  FileDef *ifd=0;
+  FileDef *ifd=nullptr;
   if (!incName.isEmpty())
   {
     FileName *fn = Doxygen::inputNameLinkedMap->find(incName);
@@ -629,7 +719,7 @@ void ClangTUParser::linkIdentifier(OutputCodeList &ol,const FileDef *fd,
   CXString usr = clang_getCursorUSR(c);
   const char *usrStr = clang_getCString(usr);
 
-  const Definition *d = 0;
+  const Definition *d = nullptr;
   auto kv = Doxygen::clangUsrMap->find(usrStr);
   if (kv!=Doxygen::clangUsrMap->end())
   {
@@ -646,8 +736,10 @@ void ClangTUParser::linkIdentifier(OutputCodeList &ol,const FileDef *fd,
   //  printf("found definition for '%s' usr='%s' name='%s'\n",
   //      text,usrStr,d->name().data());
   //}
+
   if (d && d->isLinkable())
   {
+    //printf("linkIdentifier(%s) p->insideBody=%d p->currentMemberDef=%p\n",text,p->insideBody,(void*)p->currentMemberDef);
     if (p->insideBody &&
         p->currentMemberDef && d->definitionType()==Definition::TypeMember &&
         (p->currentMemberDef!=d || p->currentLine<line)) // avoid self-reference
@@ -697,29 +789,30 @@ void ClangTUParser::detectFunctionBody(const char *s)
 void ClangTUParser::writeSources(OutputCodeList &ol,const FileDef *fd)
 {
   // (re)set global parser state
-  p->currentMemberDef=0;
+  p->currentMemberDef=nullptr;
   p->currentLine=0;
   p->searchForBody=FALSE;
   p->insideBody=FALSE;
   p->bracketCount=0;
+  p->foldStack.clear();
 
   unsigned int line=1,column=1;
   QCString lineNumber,lineAnchor;
   bool inlineCodeFragment = false;
-  ol.startCodeLine(line);
   writeLineNumber(ol,fd,line,!inlineCodeFragment);
+  ol.startCodeLine(line);
   for (unsigned int i=0;i<p->numTokens;i++)
   {
     CXSourceLocation start = clang_getTokenLocation(p->tu, p->tokens[i]);
-    unsigned int l, c;
-    clang_getSpellingLocation(start, 0, &l, &c, 0);
+    unsigned int l=0, c=0;
+    clang_getSpellingLocation(start, nullptr, &l, &c, nullptr);
     if (l > line) column = 1;
     while (line<l)
     {
       line++;
       ol.endCodeLine();
-      ol.startCodeLine(line);
       writeLineNumber(ol,fd,line,!inlineCodeFragment);
+      ol.startCodeLine(line);
     }
     while (column<c) { ol.codify(" "); column++; }
     CXString tokenString = clang_getTokenSpelling(p->tu, p->tokens[i]);
@@ -789,9 +882,9 @@ void ClangTUParser::writeSources(OutputCodeList &ol,const FileDef *fd)
                )
             {
               linkIdentifier(ol,fd,line,column,s,i);
-              if (Doxygen::searchIndex)
+              if (Doxygen::searchIndex.enabled())
               {
-                Doxygen::searchIndex->addWord(s,FALSE);
+                Doxygen::searchIndex.addWord(s,FALSE);
               }
             }
             else
@@ -804,6 +897,14 @@ void ClangTUParser::writeSources(OutputCodeList &ol,const FileDef *fd)
     clang_disposeString(tokenString);
   }
   ol.endCodeLine();
+  if (Config_getBool(HTML_CODE_FOLDING))
+  {
+    while (!p->foldStack.empty())
+    {
+      ol.endFold();
+      p->foldStack.pop_back();
+    }
+  }
   p->tooltipManager.writeTooltips(ol);
 }
 
