@@ -46,7 +46,6 @@
 #include "filedef.h"
 #include "dirdef.h"
 #include "pagedef.h"
-#include "bufstr.h"
 #include "reflist.h"
 #include "utf8.h"
 #include "indexlist.h"
@@ -70,6 +69,13 @@ class DeepCopyUniquePtr : public std::unique_ptr<T>
       if (*this!=other) this->reset(other ? new T(*other) : nullptr);
       return *this;
     }
+    DeepCopyUniquePtr(DeepCopyUniquePtr &&other) : std::unique_ptr<T>(std::move(other)) {}
+    DeepCopyUniquePtr &operator=(DeepCopyUniquePtr &&other)
+    {
+      std::unique_ptr<T>::operator=(std::move(other));
+      return *this;
+    }
+   ~DeepCopyUniquePtr() = default;
 };
 
 //! Helper to create an object wrapped in a DeepCopyUniquePtr.
@@ -86,7 +92,7 @@ class DefinitionImpl::IMPL
     void init(const QCString &df, const QCString &n);
     void setDefFileName(const QCString &df);
 
-    Definition *def = 0;
+    Definition *def = nullptr;
 
     SectionRefs sectionRefs;
 
@@ -110,14 +116,15 @@ class DefinitionImpl::IMPL
     bool hidden = FALSE;
     bool isArtificial = FALSE;
     bool isAnonymous = FALSE;
+    bool isExported = FALSE;
 
-    Definition *outerScope = 0;  // not owner
+    Definition *outerScope = nullptr;  // not owner
 
     // where the item was defined
     QCString defFileName;
     QCString defFileExt;
 
-    SrcLangExt lang = SrcLangExt_Unknown;
+    SrcLangExt lang = SrcLangExt::Unknown;
 
     QCString id; // clang unique id
 
@@ -166,7 +173,8 @@ void DefinitionImpl::IMPL::init(const QCString &df, const QCString &n)
   outerScope      = Doxygen::globalScope;
   hidden          = FALSE;
   isArtificial    = FALSE;
-  lang            = SrcLangExt_Unknown;
+  isExported      = FALSE;
+  lang            = SrcLangExt::Unknown;
 }
 
 void DefinitionImpl::setDefFile(const QCString &df,int defLine,int defCol)
@@ -280,6 +288,15 @@ DefinitionImpl::DefinitionImpl(const DefinitionImpl &d)
   if (m_impl->isSymbol) addToMap(m_impl->name,m_impl->def);
 }
 
+DefinitionImpl &DefinitionImpl::operator=(const DefinitionImpl &other)
+{
+  if (this!=&other)
+  {
+    m_impl = std::make_unique<DefinitionImpl::IMPL>(*other.m_impl);
+  }
+  return *this;
+}
+
 DefinitionImpl::~DefinitionImpl()
 {
   if (m_impl->isSymbol)
@@ -323,11 +340,11 @@ void DefinitionImpl::addSectionsToDefinition(const std::vector<const SectionInfo
     SectionManager &sm = SectionManager::instance();
     SectionInfo *gsi=sm.find(si->label());
     //printf("===== label=%s gsi=%p\n",qPrint(si->label),gsi);
-    if (gsi==0)
+    if (gsi==nullptr)
     {
       gsi = sm.add(*si);
     }
-    if (m_impl->sectionRefs.find(gsi->label())==0)
+    if (m_impl->sectionRefs.find(gsi->label())==nullptr)
     {
       m_impl->sectionRefs.add(gsi);
       gsi->setDefinition(m_impl->def);
@@ -342,7 +359,7 @@ bool DefinitionImpl::hasSections() const
   if (m_impl->sectionRefs.empty()) return FALSE;
   for (const SectionInfo *si : m_impl->sectionRefs)
   {
-    if (isSection(si->type()))
+    if (si->type().isSection())
     {
       return TRUE;
     }
@@ -350,56 +367,6 @@ bool DefinitionImpl::hasSections() const
   return FALSE;
 }
 
-void DefinitionImpl::addSectionsToIndex()
-{
-  if (m_impl->sectionRefs.empty()) return;
-  //printf("DefinitionImpl::addSectionsToIndex()\n");
-  int level=1;
-  for (auto it = m_impl->sectionRefs.begin(); it!=m_impl->sectionRefs.end(); ++it)
-  {
-    const SectionInfo *si = *it;
-    SectionType type = si->type();
-    if (isSection(type))
-    {
-      //printf("  level=%d title=%s\n",level,qPrint(si->title));
-      int nextLevel = static_cast<int>(type);
-      int i;
-      if (nextLevel>level)
-      {
-        for (i=level;i<nextLevel;i++)
-        {
-          Doxygen::indexList->incContentsDepth();
-        }
-      }
-      else if (nextLevel<level)
-      {
-        for (i=nextLevel;i<level;i++)
-        {
-          Doxygen::indexList->decContentsDepth();
-        }
-      }
-      QCString title = si->title();
-      if (title.isEmpty()) title = si->label();
-      // determine if there is a next level inside this item, but be aware of the anchor and table section references.
-      auto it_next = std::next(it);
-      bool isDir = (it_next!=m_impl->sectionRefs.end()) ?
-                       (isSection((*it_next)->type()) && static_cast<int>((*it_next)->type()) > nextLevel) : FALSE;
-      Doxygen::indexList->addContentsItem(isDir,title,
-                                         getReference(),
-                                         m_impl->def->getOutputFileBase(),
-                                         si->label(),
-                                         FALSE,
-                                         TRUE);
-      level = nextLevel;
-    }
-  }
-  while (level>1)
-  {
-    Doxygen::indexList->decContentsDepth();
-    level--;
-  }
-}
-
 void DefinitionImpl::writeDocAnchorsToTagFile(TextStream &tagFile) const
 {
   if (!m_impl->sectionRefs.empty())
@@ -431,7 +398,7 @@ bool DefinitionImpl::_docsAlreadyAdded(const QCString &doc,QCString &sigList)
   // to avoid mismatches due to differences in indenting, we first remove
   // double whitespaces...
   QCString docStr = doc.simplifyWhiteSpace();
-  MD5Buffer(docStr.data(),docStr.length(),md5_sig);
+  MD5Buffer(docStr.data(),static_cast<unsigned int>(docStr.length()),md5_sig);
   MD5SigToString(md5_sig,sigStr);
   //printf("%s:_docsAlreadyAdded doc='%s' sig='%s' docSigs='%s'\n",
   //    qPrint(name()),qPrint(doc),qPrint(sigStr),qPrint(sigList));
@@ -505,7 +472,7 @@ void DefinitionImpl::_setBriefDescription(const QCString &b,const QCString &brie
   brief = stripLeadingAndTrailingEmptyLines(brief,briefLine);
   brief = brief.stripWhiteSpace();
   if (brief.isEmpty()) return;
-  uint32_t bl = brief.length();
+  size_t bl = brief.length();
   if (bl>0)
   {
     if (!theTranslator || theTranslator->needsPunctuation()) // add punctuation if needed
@@ -604,9 +571,8 @@ class FilterCache
     //! buffer \a str. Applies filtering if FILTER_SOURCE_FILES is enabled and the file extension
     //! matches a filter. Caches file information so that subsequent extraction of blocks from
     //! the same file can be performed efficiently
-    bool getFileContents(const QCString &fileName,size_t startLine,size_t endLine, BufStr &str)
+    bool getFileContents(const QCString &fileName,size_t startLine,size_t endLine, std::string &str)
     {
-      std::lock_guard<std::mutex> lock(m_mutex);
       bool filterSourceFiles = Config_getBool(FILTER_SOURCE_FILES);
       QCString filter = getFileFilter(fileName,TRUE);
       bool usePipe = !filter.isEmpty() && filterSourceFiles;
@@ -615,11 +581,13 @@ class FilterCache
     }
   private:
     bool getFileContentsPipe(const QCString &fileName,const QCString &filter,
-                             size_t startLine,size_t endLine,BufStr &str)
+                             size_t startLine,size_t endLine,std::string &str)
     {
+      std::unique_lock<std::mutex> lock(m_mutex);
       auto it = m_cache.find(fileName.str());
       if (it!=m_cache.end()) // cache hit: reuse stored result
       {
+        lock.unlock();
         auto item = it->second;
         //printf("getFileContents(%s): cache hit\n",qPrint(fileName));
         // file already processed, get the results after filtering from the tmp file
@@ -642,21 +610,19 @@ class FilterCache
         QCString cmd=filter+" \""+fileName+"\"";
         Debug::print(Debug::ExtCmd,0,"Executing popen(`%s`)\n",qPrint(cmd));
         FILE *f = Portable::popen(cmd,"r");
-        if (f==0)
+        if (f==nullptr)
         {
           // handle error
           err("Error opening filter pipe command '%s'\n",qPrint(cmd));
-          str.addChar('\0');
           return false;
         }
         FILE *bf = Portable::fopen(Doxygen::filterDBFileName,"a+b");
         FilterCacheItem item;
         item.filePos = m_endPos;
-        if (bf==0)
+        if (bf==nullptr)
         {
           // handle error
           err("Error opening filter database file %s\n",qPrint(Doxygen::filterDBFileName));
-          str.addChar('\0');
           Portable::pclose(f);
           return false;
         }
@@ -673,15 +639,13 @@ class FilterCache
             // handle error
             err("Failed to write to filter database %s. Wrote %zu out of %zu bytes\n",
                 qPrint(Doxygen::filterDBFileName),bytesWritten,bytesRead);
-            str.addChar('\0');
             Portable::pclose(f);
             fclose(bf);
             return false;
           }
           size+=bytesWritten;
-          str.addArray(buf,static_cast<uint32_t>(bytesWritten));
+          str+=std::string_view(buf,bytesWritten);
         }
-        str.addChar('\0');
         item.fileSize = size;
         // add location entry to the dictionary
         m_cache.insert(std::make_pair(fileName.str(),item));
@@ -700,8 +664,9 @@ class FilterCache
 
     //! reads the fragment start at \a startLine and ending at \a endLine from file \a fileName
     //! into buffer \a str
-    bool getFileContentsDisk(const QCString &fileName,size_t startLine,size_t endLine,BufStr &str)
+    bool getFileContentsDisk(const QCString &fileName,size_t startLine,size_t endLine,std::string &str)
     {
+      std::unique_lock<std::mutex> lock(m_mutex);
       // normal file
       //printf("getFileContents(%s): no filter\n",qPrint(fileName));
       auto it = m_lineOffsets.find(fileName.str());
@@ -714,6 +679,7 @@ class FilterCache
       }
       else // file already processed before
       {
+        lock.unlock();
         auto [ startLineOffset, fragmentSize] = getFragmentLocation(it->second,startLine,endLine);
         //printf("%s: existing file [%zu-%zu] -> start=%zu size=%zu\n",
         //    qPrint(fileName),startLine,endLine,startLineOffset,fragmentSize);
@@ -724,14 +690,14 @@ class FilterCache
 
     //! computes the starting offset for each line for file \a fileName, whose contents should
     //! already be stored in buffer \a str.
-    void compileLineOffsets(const QCString &fileName,const BufStr &str)
+    void compileLineOffsets(const QCString &fileName,const std::string &str)
     {
-      const char *p=str.data();
       // line 1 (index 0) is at offset 0
       auto it = m_lineOffsets.insert(std::make_pair(fileName.data(),LineOffsets{0})).first;
+      const char *p=str.data();
       while (*p)
       {
-        char c;
+        char c=0;
         while ((c=*p)!='\n' && c!=0) p++; // search until end of the line
         p++;
         it->second.push_back(p-str.data());
@@ -754,7 +720,7 @@ class FilterCache
 
     //! Shrinks buffer \a str which should hold the contents of \a fileName to the
     //! fragment starting a line \a startLine and ending at line \a endLine
-    void shrinkBuffer(BufStr &str,const QCString &fileName,size_t startLine,size_t endLine)
+    void shrinkBuffer(std::string &str,const QCString &fileName,size_t startLine,size_t endLine)
     {
       // compute offsets from start for each line
       compileLineOffsets(fileName,str);
@@ -764,23 +730,20 @@ class FilterCache
       auto [ startLineOffset, fragmentSize] = getFragmentLocation(lineOffsets,startLine,endLine);
       //printf("%s: new file [%zu-%zu]->[%zu-%zu] size=%zu\n",
       //    qPrint(fileName),startLine,endLine,startLineOffset,endLineOffset,fragmentSize);
-      str.dropFromStart(startLineOffset);
-      str.resize(fragmentSize+1);
-      str.at(fragmentSize)='\0';
+      str.erase(0,startLineOffset);
+      str.resize(fragmentSize);
     }
 
     //! Reads the fragment start at byte offset \a startOffset of file \a fileName into buffer \a str.
     //! Result will be a null terminated. If size==0 the whole file will be read and startOffset is ignored.
     //! If size>0, size bytes will be read.
-    void readFragmentFromFile(BufStr &str,const QCString &fileName,size_t startOffset,size_t size=0)
+    void readFragmentFromFile(std::string &str,const QCString &fileName,size_t startOffset,size_t size=0)
     {
       std::ifstream ifs = Portable::openInputStream(fileName,true,true);
       if (size==0) { startOffset=0; size = static_cast<size_t>(ifs.tellg()); }
       ifs.seekg(startOffset, std::ios::beg);
-      str.resize(size+1);
+      str.resize(size);
       ifs.read(str.data(), size);
-      str.skip(size);
-      str.addChar('\0');
     }
 
     FilterCache() : m_endPos(0) { }
@@ -810,7 +773,7 @@ FilterCache &FilterCache::instance()
  * The line actually containing the bracket is returned via endLine.
  * Note that for VHDL code the bracket search is not done.
  */
-bool readCodeFragment(const QCString &fileName,
+bool readCodeFragment(const QCString &fileName,bool isMacro,
                       int &startLine,int &endLine,QCString &result)
 {
   bool filterSourceFiles = Config_getBool(FILTER_SOURCE_FILES);
@@ -819,15 +782,16 @@ bool readCodeFragment(const QCString &fileName,
   int tabSize = Config_getInt(TAB_SIZE);
   SrcLangExt lang = getLanguageFromFileName(fileName);
   const int blockSize = 4096;
-  BufStr str(blockSize);
+  std::string str;
   FilterCache::instance().getFileContents(fileName,
                                           static_cast<size_t>(std::max(1,startLine)),
                                           static_cast<size_t>(std::max({1,startLine,endLine})),str);
   //printf("readCodeFragment(%s,startLine=%d,endLine=%d)=\n[[[\n%s]]]\n",qPrint(fileName),startLine,endLine,qPrint(str));
 
-  bool found = lang==SrcLangExt_VHDL   ||
-               lang==SrcLangExt_Python ||
-               lang==SrcLangExt_Fortran;
+  bool found = lang==SrcLangExt::VHDL   ||
+               lang==SrcLangExt::Python ||
+               lang==SrcLangExt::Fortran ||
+               isMacro;
                // for VHDL, Python, and Fortran no bracket search is possible
   char *p=str.data();
   if (p && *p)
@@ -904,7 +868,7 @@ bool readCodeFragment(const QCString &fileName,
       do
       {
         //printf("reading line %d in range %d-%d\n",lineNr,startLine,endLine);
-        int size_read;
+        int size_read=0;
         do
         {
           // read up to maxLineLength-1 bytes, the last byte being zero
@@ -926,7 +890,7 @@ bool readCodeFragment(const QCString &fileName,
       int braceIndex   = result.findRev('}');
       if (braceIndex > newLineIndex)
       {
-        result.truncate(static_cast<size_t>(braceIndex+1));
+        result.resize(static_cast<size_t>(braceIndex+1));
       }
       endLine=lineNr-1;
     }
@@ -936,7 +900,22 @@ bool readCodeFragment(const QCString &fileName,
       Debug::print(Debug::FilterOutput,0,"-------------\n%s\n-------------\n",qPrint(result));
     }
   }
-  result = transcodeCharacterStringToUTF8(getEncoding(FileInfo(fileName.str())),result);
+  QCString encoding = getEncoding(FileInfo(fileName.str()));
+  if (encoding!="UTF-8")
+  {
+    std::string encBuf = result.str();
+    bool ok = transcodeCharacterStringToUTF8(encBuf,encoding.data());
+    if (ok)
+    {
+      result = QCString(encBuf);
+    }
+    else
+    {
+      err("failed to transcode characters in code fragment in file %s lines %d to %d, from input encoding %s to UTF-8\n",
+          qPrint(fileName),startLine,endLine,qPrint(encoding));
+
+    }
+  }
   if (!result.isEmpty() && result.at(result.length()-1)!='\n') result += "\n";
   //printf("readCodeFragment(%d-%d)=%s\n",startLine,endLine,qPrint(result));
   return found;
@@ -1049,28 +1028,29 @@ bool DefinitionImpl::hasSources() const
 /*! Write code of this definition into the documentation */
 void DefinitionImpl::writeInlineCode(OutputList &ol,const QCString &scopeName) const
 {
-  bool inlineSources = Config_getBool(INLINE_SOURCES);
-  ol.pushGeneratorState();
-  //printf("Source Fragment %s: %d-%d bodyDef=%p\n",qPrint(name()),
-  //        m_startBodyLine,m_endBodyLine,m_bodyDef);
+  const MemberDef *thisMd = nullptr;
+  if (m_impl->def->definitionType()==Definition::TypeMember)
+  {
+    thisMd = toMemberDef(m_impl->def);
+  }
+  bool inlineSources = thisMd && thisMd->hasInlineSource();
+  //printf("Source Fragment %s: %d-%d\n",qPrint(name()),
+  //        m_impl->body->startLine,m_impl->body->endLine);
   if (inlineSources && hasSources())
   {
+    ol.pushGeneratorState();
     QCString codeFragment;
+    bool isMacro = thisMd && thisMd->memberType()==MemberType::Define;
     int actualStart=m_impl->body->startLine,actualEnd=m_impl->body->endLine;
-    if (readCodeFragment(m_impl->body->fileDef->absFilePath(),
+    if (readCodeFragment(m_impl->body->fileDef->absFilePath(),isMacro,
           actualStart,actualEnd,codeFragment)
        )
     {
-      //printf("Adding code fragment '%s' ext='%s'\n",
-      //    qPrint(codeFragment),qPrint(m_impl->defFileExt));
+      //printf("Adding code fragment '%s' ext='%s' range=%d-%d\n",
+      //    qPrint(codeFragment),qPrint(m_impl->defFileExt),actualStart,actualEnd);
       auto intf = Doxygen::parserManager->getCodeParser(m_impl->defFileExt);
       intf->resetCodeParserState();
       //printf("Read:\n'%s'\n\n",qPrint(codeFragment));
-      const MemberDef *thisMd = 0;
-      if (m_impl->def->definitionType()==Definition::TypeMember)
-      {
-        thisMd = toMemberDef(m_impl->def);
-      }
 
       auto &codeOL = ol.codeGenerators();
       codeOL.startCodeFragment("DoxyCode");
@@ -1089,8 +1069,8 @@ void DefinitionImpl::writeInlineCode(OutputList &ol,const QCString &scopeName) c
                      );
       codeOL.endCodeFragment("DoxyCode");
     }
+    ol.popGeneratorState();
   }
-  ol.popGeneratorState();
 }
 
 static inline MemberVector refMapToVector(const std::unordered_map<std::string,MemberDef *> &map)
@@ -1103,7 +1083,7 @@ static inline MemberVector refMapToVector(const std::unordered_map<std::string,M
                  { return item.second; }     // extract value to add from map Key,Value pair
                 );
   // and sort it
-  std::sort(result.begin(),result.end(),
+  std::stable_sort(result.begin(),result.end(),
               [](const auto &m1,const auto &m2) { return genericCompareMembers(m1,m2)<0; });
   return result;
 }
@@ -1223,58 +1203,25 @@ bool DefinitionImpl::hasUserDocumentation() const
   return hasDocs;
 }
 
-
-void DefinitionImpl::addSourceReferencedBy(MemberDef *md)
+void DefinitionImpl::addSourceReferencedBy(MemberDef *md,const QCString &sourceRefName)
 {
   if (md)
   {
-    QCString name  = md->name();
-    QCString scope = md->getScopeString();
-
-    if (!scope.isEmpty())
-    {
-      name.prepend(scope+"::");
-    }
-    else if (md->isStatic() && md->getFileDef())
-    {
-      name.prepend(md->getFileDef()->name()+":");
-    }
-    if (md->isCallable())
-    {
-      name.append(md->argsString());
-    }
-
-    m_impl->sourceRefByDict.insert({name.str(),md});
+    m_impl->sourceRefByDict.insert({sourceRefName.str(),md});
   }
 }
 
-void DefinitionImpl::addSourceReferences(MemberDef *md)
+void DefinitionImpl::addSourceReferences(MemberDef *md,const QCString &sourceRefName)
 {
   if (md)
   {
-    QCString name  = md->name();
-    QCString scope = md->getScopeString();
-
-    if (!scope.isEmpty())
-    {
-      name.prepend(scope+"::");
-    }
-    else if (md->isStatic() && md->getFileDef())
-    {
-      name.prepend(md->getFileDef()->name()+":");
-    }
-    if (md->isCallable())
-    {
-      name.append(md->argsString());
-    }
-
-    m_impl->sourceRefsDict.insert({name.str(),md});
+    m_impl->sourceRefsDict.insert({sourceRefName.str(),md});
   }
 }
 
 const Definition *DefinitionImpl::findInnerCompound(const QCString &) const
 {
-  return 0;
+  return nullptr;
 }
 
 void DefinitionImpl::addInnerCompound(Definition *)
@@ -1293,7 +1240,7 @@ QCString DefinitionImpl::qualifiedName() const
   }
 
   //printf("start %s::qualifiedName() localName=%s\n",qPrint(name()),qPrint(m_impl->localName));
-  if (m_impl->outerScope==0)
+  if (m_impl->outerScope==nullptr)
   {
     if (m_impl->localName=="<globalScope>")
     {
@@ -1334,7 +1281,7 @@ void DefinitionImpl::setOuterScope(Definition *d)
   }
   if (!found)
   {
-    m_impl->qualifiedName.resize(0); // flush cached scope name
+    m_impl->qualifiedName.clear(); // flush cached scope name
     m_impl->outerScope = d;
   }
   m_impl->hidden = m_impl->hidden || d->isHidden();
@@ -1365,7 +1312,7 @@ void DefinitionImpl::mergeRefItems(Definition *d)
                                 otherXrefList.begin(),otherXrefList.end());
 
   // sort results on itemId
-  std::sort(m_impl->xrefListItems.begin(),m_impl->xrefListItems.end(),
+  std::stable_sort(m_impl->xrefListItems.begin(),m_impl->xrefListItems.end(),
             [](RefItem *left,RefItem *right)
             { return  left->id() <right->id() ||
                      (left->id()==right->id() &&
@@ -1541,7 +1488,9 @@ int DefinitionImpl::docLine() const
 
 QCString DefinitionImpl::docFile() const
 {
-  return m_impl->details ? m_impl->details->file : m_impl->brief ? m_impl->brief->file : QCString("<"+m_impl->name+">");
+  if (m_impl->details && !m_impl->details->file.isEmpty()) return m_impl->details->file;
+  else if (m_impl->brief && !m_impl->brief->file.isEmpty()) return m_impl->brief->file;
+  else  return "<" + m_impl->name + ">";
 }
 
 //----------------------------------------------------------------------------
@@ -1604,7 +1553,7 @@ void DefinitionImpl::computeTooltip()
 {
   if (m_impl->brief && m_impl->brief->tooltip.isEmpty() && !m_impl->brief->doc.isEmpty())
   {
-    const MemberDef *md = m_impl->def->definitionType()==Definition::TypeMember ? toMemberDef(m_impl->def) : 0;
+    const MemberDef *md = m_impl->def->definitionType()==Definition::TypeMember ? toMemberDef(m_impl->def) : nullptr;
     const Definition *scope = m_impl->def->definitionType()==Definition::TypeMember ? m_impl->def->getOuterScope() : m_impl->def;
     m_impl->brief->tooltip = parseCommentAsText(scope,md,
                                 m_impl->brief->doc, m_impl->brief->file, m_impl->brief->line);
@@ -1623,7 +1572,7 @@ int DefinitionImpl::briefLine() const
 
 QCString DefinitionImpl::briefFile() const
 {
-  return m_impl->brief ? m_impl->brief->file : QCString("<"+m_impl->name+">");
+  return m_impl->brief && !m_impl->brief->file.isEmpty() ? m_impl->brief->file : QCString("<"+m_impl->name+">");
 }
 
 //----------------------
@@ -1640,7 +1589,7 @@ int DefinitionImpl::inbodyLine() const
 
 QCString DefinitionImpl::inbodyFile() const
 {
-  return m_impl->inbodyDocs ? m_impl->inbodyDocs->file : QCString("<"+m_impl->name+">");
+  return m_impl->inbodyDocs && !m_impl->inbodyDocs->file.isEmpty() ? m_impl->inbodyDocs->file : QCString("<"+m_impl->name+">");
 }
 
 
@@ -1676,6 +1625,11 @@ bool DefinitionImpl::isArtificial() const
   return m_impl->isArtificial;
 }
 
+bool DefinitionImpl::isExported() const
+{
+  return m_impl->isExported;
+}
+
 QCString DefinitionImpl::getReference() const
 {
   return m_impl->ref;
@@ -1703,7 +1657,7 @@ int DefinitionImpl::getEndBodyLine() const
 
 const FileDef *DefinitionImpl::getBodyDef() const
 {
-  return m_impl->body ? m_impl->body->fileDef : 0;
+  return m_impl->body ? m_impl->body->fileDef : nullptr;
 }
 
 const GroupList &DefinitionImpl::partOfGroups() const
@@ -1800,6 +1754,11 @@ void DefinitionImpl::setArtificial(bool b)
   m_impl->isArtificial = b;
 }
 
+void DefinitionImpl::setExported(bool b)
+{
+  m_impl->isExported = b;
+}
+
 void DefinitionImpl::setLocalName(const QCString &name)
 {
   m_impl->localName=name;
@@ -1836,7 +1795,7 @@ QCString DefinitionImpl::externalReference(const QCString &relPath) const
     if (it!=Doxygen::tagDestinationMap.end())
     {
       QCString result(it->second);
-      uint32_t l = result.length();
+      size_t l = result.length();
       if (!relPath.isEmpty() && l>0 && result.at(0)=='.')
       { // relative path -> prepend relPath.
         result.prepend(relPath);
@@ -1892,6 +1851,16 @@ void DefinitionAliasImpl::init()
 {
   //printf("%s::addToMap(%s)\n",qPrint(name()),qPrint(alias->name()));
   addToMap(m_symbolName,m_def);
+  if (m_scope==nullptr)
+  {
+    m_qualifiedName = m_def->localName();
+  }
+  else
+  {
+    m_qualifiedName = m_scope->qualifiedName()+
+      getLanguageSpecificSeparator(m_scope->getLanguage())+
+      m_def->localName();
+  }
 }
 
 void DefinitionAliasImpl::deinit()
@@ -1899,34 +1868,13 @@ void DefinitionAliasImpl::deinit()
   removeFromMap(m_symbolName,m_def);
 }
 
-void DefinitionAliasImpl::updateQualifiedName() const
-{
-  std::lock_guard<std::recursive_mutex> lock(g_qualifiedNameMutex);
-  if (m_qualifiedName.isEmpty())
-  {
-    //printf("start %s::qualifiedName() localName=%s\n",qPrint(name()),qPrint(m_impl->localName));
-    if (m_scope==0)
-    {
-      m_qualifiedName = m_def->localName();
-    }
-    else
-    {
-      m_qualifiedName = m_scope->qualifiedName()+
-        getLanguageSpecificSeparator(m_scope->getLanguage())+
-        m_def->localName();
-    }
-  }
-}
-
 QCString DefinitionAliasImpl::qualifiedName() const
 {
-  updateQualifiedName();
   return m_qualifiedName;
 }
 
 const QCString &DefinitionAliasImpl::name() const
 {
-  updateQualifiedName();
   return m_qualifiedName;
 }
 
@@ -1934,13 +1882,13 @@ const QCString &DefinitionAliasImpl::name() const
 
 Definition *toDefinition(DefinitionMutable *dm)
 {
-  if (dm==0) return 0;
+  if (dm==nullptr) return nullptr;
   return dm->toDefinition_();
 }
 
 DefinitionMutable *toDefinitionMutable(Definition *d)
 {
-  if (d==0) return 0;
+  if (d==nullptr) return nullptr;
   return d->toDefinitionMutable_();
 }
 
