@@ -66,6 +66,8 @@ static void startSimpleSect(TextStream &t,const DocSimpleSect &s)
       t << "remark"; break;
     case DocSimpleSect::Attention:
       t << "attention"; break;
+    case DocSimpleSect::Important:
+      t << "important"; break;
     case DocSimpleSect::User:
       t << "par"; break;
     case DocSimpleSect::Rcs:
@@ -147,7 +149,7 @@ static void visitPostEnd(TextStream &t, const char *cmd)
 
 XmlDocVisitor::XmlDocVisitor(TextStream &t,OutputCodeList &ci,const QCString &langExt)
   : m_t(t), m_ci(ci), m_insidePre(FALSE), m_hide(FALSE),
-    m_langExt(langExt)
+    m_langExt(langExt), m_sectionLevel(0)
 {
 }
 
@@ -413,7 +415,7 @@ void XmlDocVisitor::operator()(const DocInclude &inc)
                                            -1,    // start line
                                            -1,    // end line
                                            FALSE, // inline fragment
-                                           0,     // memberDef
+                                           nullptr,     // memberDef
                                            TRUE   // show line numbers
 					   );
          m_t << "</programlisting>";
@@ -426,11 +428,11 @@ void XmlDocVisitor::operator()(const DocInclude &inc)
                                         langExt,
                                         inc.isExample(),
                                         inc.exampleFile(),
-                                        0,     // fileDef
+                                        nullptr,     // fileDef
                                         -1,    // startLine
                                         -1,    // endLine
                                         TRUE,  // inlineFragment
-                                        0,     // memberDef
+                                        nullptr,     // memberDef
                                         FALSE  // show line numbers
 				       );
       m_t << "</programlisting>";
@@ -529,7 +531,7 @@ void XmlDocVisitor::operator()(const DocIncOperator &op)
                                           op.line(),    // startLine
                                           -1,    // endLine
                                           FALSE, // inline fragment
-                                          0,     // memberDef
+                                          nullptr,     // memberDef
                                           op.showLineNo()  // show line numbers
                                          );
     }
@@ -613,7 +615,19 @@ void XmlDocVisitor::operator()(const DocAutoList &l)
 void XmlDocVisitor::operator()(const DocAutoListItem &li)
 {
   if (m_hide) return;
-  m_t << "<listitem>";
+  switch (li.itemNumber())
+  {
+    case DocAutoList::Unchecked: // unchecked
+      m_t << "<listitem override=\"unchecked\">";
+      break;
+    case DocAutoList::Checked_x: // checked with x
+    case DocAutoList::Checked_X: // checked with X
+      m_t << "<listitem override=\"checked\">";
+      break;
+    default:
+      m_t << "<listitem>";
+      break;
+  }
   visitChildren(li);
   m_t << "</listitem>";
 }
@@ -673,15 +687,29 @@ void XmlDocVisitor::operator()(const DocSimpleListItem &li)
 void XmlDocVisitor::operator()(const DocSection &s)
 {
   if (m_hide) return;
-  m_t << "<sect" << s.level() << " id=\"" << s.file();
-  if (!s.anchor().isEmpty()) m_t << "_1" << s.anchor();
-  m_t << "\">\n";
+  int orgSectionLevel = m_sectionLevel;
+  QCString sectId = s.file();
+  if (!s.anchor().isEmpty()) sectId += "_1"+s.anchor();
+  while (m_sectionLevel+1<s.level()) // fix missing intermediate levels
+  {
+    m_sectionLevel++;
+    m_t << "<sect" << m_sectionLevel << " id=\"" << sectId << "_1s" << m_sectionLevel << "\">";
+  }
+  m_sectionLevel++;
+  m_t << "<sect" << s.level() << " id=\"" << sectId << "\">\n";
   if (s.title())
   {
     std::visit(*this,*s.title());
   }
   visitChildren(s);
-  m_t << "</sect" << s.level() << ">\n";
+  m_t << "</sect" << s.level() << ">";
+  m_sectionLevel--;
+  while (orgSectionLevel<m_sectionLevel) // fix missing intermediate levels
+  {
+    m_t << "</sect" << m_sectionLevel << ">";
+    m_sectionLevel--;
+  }
+  m_t << "\n";
 }
 
 void XmlDocVisitor::operator()(const DocHtmlList &s)
@@ -914,7 +942,7 @@ void XmlDocVisitor::operator()(const DocImage &img)
                 altValue, img.isInlineImage());
 
   // copy the image to the output dir
-  FileDef *fd = 0;
+  FileDef *fd = nullptr;
   bool ambig;
   if (url.isEmpty() && (fd=findFileDef(Doxygen::imageNameLinkedMap,img.name(),ambig)))
   {
